@@ -36,3 +36,15 @@ class DuplicateResourceKey(Exception):
 
 class InvalidResourceKey(Exception):
     pass
+
+
+class MissingEpubResource(Exception):
+    pass
+
+
+class UnknownTxtEncoding(Exception):
+    pass
+
+
+class MetsTagNotFound(Exception):
+    pass
