@@ -11,7 +11,7 @@ from bs4 import BeautifulSoup
 from ebooklib import epub
 from ebooklib.epub import EpubBook
 
-from ..exceptions import NotLoadedOrEmpty
+from ..exceptions import NotLoadedOrEmpty, MissingEpubResource
 from ..factory.resource_manager import ResourceManager
 from ..handlers.base_handler import FileHandler
 from ..tools.image_classification import ImageClassificator
@@ -93,10 +93,15 @@ class EPUBHandler(FileHandler):
                     # Add dummy toc.ncx
                     zout.writestr("toc.ncx", dummy_ncx)
 
-            epub_output = epub.read_epub(str(output_path), {"ignore_ncx": True})
+            try:
+                epub_output = epub.read_epub(str(output_path), {"ignore_ncx": True})
+            except KeyError as e:
+                raise MissingEpubResource(e)
+
             return epub_output
         except Exception as e:
             logger.error(f"Error reading EPUB file: {e}")
+            raise
 
     def _read_epub(self) -> EpubBook:
         """
