@@ -2,8 +2,8 @@ import logging
 import os
 import warnings
 from collections import defaultdict
-from urllib.parse import urlparse
 from typing import List
+from urllib.parse import urlparse
 
 import numpy as np
 from PIL import Image
@@ -16,6 +16,7 @@ from rara_digitizer.utils.constants import (
     METSALTO_IMG_LABELS,
     KEEP_METSALTO_IMG_LABELS,
 )
+from ..exceptions import MetsTagNotFound
 from ..factory.resource_manager import ResourceManager
 from ..handlers.base_handler import FileHandler
 from ..tools.image_classification import ImageClassificator
@@ -29,11 +30,11 @@ warnings.filterwarnings("ignore", category=FutureWarning)
 
 class METSALTOHandler(FileHandler):
     def __init__(
-        self,
-        mets_folder_path: str,
-        mets_file_path: str,
-        resource_manager: ResourceManager,
-        **kwargs,
+            self,
+            mets_folder_path: str,
+            mets_file_path: str,
+            resource_manager: ResourceManager,
+            **kwargs,
     ) -> None:
         """
         Initializes the METSALTOHandler by loading the METS content into `self.document` and ALTO content into
@@ -66,7 +67,7 @@ class METSALTOHandler(FileHandler):
         self.estimate_page_count = False
 
         self.start_page = (
-            max(1, kwargs.get("start_page", 1)) - 1
+                max(1, kwargs.get("start_page", 1)) - 1
         )  # Ensure start_page is indexed from 0
         self.end_page = (
             self.start_page + kwargs.get("max_pages", 0)
@@ -146,6 +147,7 @@ class METSALTOHandler(FileHandler):
             mets_tag = soup.find("mets")
             if mets_tag is None:
                 logger.error("METS file is invalid: mets tag not found.")
+                raise MetsTagNotFound("Could not find mets tag!")
             return soup
 
         except Exception as e:
@@ -174,7 +176,7 @@ class METSALTOHandler(FileHandler):
                 "fileGrp",
                 {
                     "ID": lambda x: x
-                    and any(
+                                    and any(
                         pattern.lower() in x.lower()
                         for pattern in self.mets_filegroup_patterns[group_name]
                     )
@@ -182,8 +184,8 @@ class METSALTOHandler(FileHandler):
             )
             if file_group:
                 for file in file_group.find_all("file")[
-                    self.start_page : self.end_page
-                ]:
+                            self.start_page: self.end_page
+                            ]:
                     file_id = file.get("ID")
                     href = file.find("FLocat").get("xlink:href")
                     href = href.replace("file://", "")
@@ -278,7 +280,7 @@ class METSALTOHandler(FileHandler):
             logger.error(f"Error extracting text from the METS/ALTO file: {e}")
 
     def _calculate_mean_wc_from_soups(
-        self, soup_objects: list[BeautifulSoup]
+            self, soup_objects: list[BeautifulSoup]
     ) -> float | None:
         """
         Calculate the mean of the word count (WC) attribute values for all
@@ -339,7 +341,7 @@ class METSALTOHandler(FileHandler):
         return alto_refs
 
     def _extract_text_from_block(
-        self, alto_id: str, block_tag: str, carried_word: str
+            self, alto_id: str, block_tag: str, carried_word: str
     ) -> tuple[str, str]:
         """
         Extracts text from the referenced block, handling hyphenated words.
@@ -498,13 +500,13 @@ class METSALTOHandler(FileHandler):
                 section_meta = self.document.find("dmdSec", {"ID": section_id})
             else:
                 section_meta = None
-                
+
             section["section_meta"] = section_meta
-            
+
         return document_content
 
     def _process_single_div(
-        self, div: Tag, section_id: str, section_type: str, document_content: list
+            self, div: Tag, section_id: str, section_type: str, document_content: list
     ):
         """
         Process a single div and append the parsed content to document_content.
@@ -571,7 +573,6 @@ class METSALTOHandler(FileHandler):
                     document_content.append(paragraph_content)
         return document_content
 
-
     def extract_page_numbers(self) -> int | None:
         """
         Extracts the total number of pages in the file and stores it in the `self.page_count` attribute.
@@ -635,7 +636,7 @@ class METSALTOHandler(FileHandler):
             logger.error(f"Error extracting physical dimension from METS/ALTO: {e}")
 
     def _process_page(
-        self, alto_page: BeautifulSoup, alto_image_path: str, alto_id: str
+            self, alto_page: BeautifulSoup, alto_image_path: str, alto_id: str
     ) -> list:
         """
         Processes a single ALTO page, extracts images and metadata (label, coordinates, page number).
@@ -663,7 +664,7 @@ class METSALTOHandler(FileHandler):
         page_nr = self.alto_id_to_page_nr[alto_id]
         page_images = []
         for block in alto_page.find_all(
-            "ComposedBlock", TYPE=lambda t: t and t.lower() in self.metsalto_img_labels
+                "ComposedBlock", TYPE=lambda t: t and t.lower() in self.metsalto_img_labels
         ):
             image_metadata = self._extract_image_block_meta(block, scale_x, scale_y)
             cropped_image = self._crop_image(
@@ -685,7 +686,7 @@ class METSALTOHandler(FileHandler):
         return page_images
 
     def _calculate_scaling_factors(
-        self, alto_page: BeautifulSoup, alto_image: Image
+            self, alto_page: BeautifulSoup, alto_image: Image
     ) -> tuple:
         """
         Calculates scaling factors based on the ALTO page and image dimensions.
@@ -712,7 +713,7 @@ class METSALTOHandler(FileHandler):
         return scale_x, scale_y
 
     def _extract_image_block_meta(
-        self, block: BeautifulSoup, scale_x: float, scale_y: float, label: str = None
+            self, block: BeautifulSoup, scale_x: float, scale_y: float, label: str = None
     ) -> dict:
         """
         Extracts image metadata from a block (label, coordinates).
@@ -777,9 +778,9 @@ class METSALTOHandler(FileHandler):
             The cropped image.
         """
         cropped_image_array = alto_image_array[
-            coords["VPOS"] : coords["VPOS"] + coords["HEIGHT"],
-            coords["HPOS"] : coords["HPOS"] + coords["WIDTH"],
-        ]
+                              coords["VPOS"]: coords["VPOS"] + coords["HEIGHT"],
+                              coords["HPOS"]: coords["HPOS"] + coords["WIDTH"],
+                              ]
         return Image.fromarray(cropped_image_array)
 
     def extract_images(self) -> list[dict[str, str | int | dict | None]]:
@@ -796,7 +797,7 @@ class METSALTOHandler(FileHandler):
 
         extracted_images = []
         for (alto_id, alto_page), (image_id, alto_image_path) in zip(
-            self.pages.items(), self.alto_image_paths.items()
+                self.pages.items(), self.alto_image_paths.items()
         ):
             page_images = self._process_page(alto_page, alto_image_path, alto_id)
             extracted_images.extend(page_images)
@@ -842,7 +843,7 @@ class METSALTOHandler(FileHandler):
         content: list
             A list of dictionaries containing document content split into sections
         """
-     
+
         for entry in content:
             for remove_key in remove_keys:
                 del entry[remove_key]
